from Globals import InitializeClass
from OFS.SimpleItem import Item
from OFS.ObjectManager import ObjectManager
from Globals import DTMLFile
from OFS.PropertyManager import PropertyManager
from AccessControl.Role import RoleManager
from AccessControl import ClassSecurityInfo
from Bloggem import Bloggem
from Products.Bloggem.Bloggem import checkReservedName
import string
from Bloggem import bloggemItemTypes


class BloggemFolder(ObjectManager, PropertyManager, RoleManager, Item, Bloggem):

    """
    BloggemFolder is a Bloggem product. BloggemFolders contain the
    BloggemItems (BloggemArticle, BloggemLink and BloggemBook) that
    appear in Bloggem.  BloggemFolders create categories and navigation
    for these BlogemItems.  BloggemFolders are contained by the Bloggem 
    root or by other BloggemFolders.

    """

    meta_type = 'Bloggem Category'
    
    manage_options = (
        {'label': 'Contents', 'action': 'manage_main'},
        {'label': 'Properties', 'action': 'manage_editFolderForm',},
        {'label': 'View', 'action': 'index_html',},
        ) + Item.manage_options 
        
    
    def __init__(self, id, title, description, meta_title, meta_description, meta_keywords, REQUEST=None):
        "Inits the product with default values"
        self.id = id
        self.title = title
        self.meta_keywords = meta_keywords
        
        if meta_title <> '':
            self.meta_title = meta_title      
        
        if meta_description <> '':
            self.meta_description = meta_description
            
        if description <> '':
            self.description = description            
            
        
    def manage_editAction(self, title, description, meta_title, meta_description, meta_keywords, RESPONSE=None):
        """Changes the product values"""
        self.title = title
        self.meta_keywords = meta_keywords
        
        # Descriptions, meta_titles, and meta_descriptions are designed to inherit if they do not exist.
        if description == '' and hasattr(self.aq_explicit, 'description'):
            delattr(self, 'description')     
        elif description <> '': 
            self.description = description
        spam2 = self.meta_title
        if meta_title == '' and hasattr(self.aq_explicit, 'meta_title'):
            delattr(self, 'meta_title')     
        elif meta_title <> '': 
            self.meta_title = meta_title
        spam = self.meta_description
        if meta_description == '' and hasattr(self.aq_explicit, 'meta_description'):
            del self.meta_description     
        elif meta_description <> '': 
            self.meta_description = meta_description
 
        self._p_changed = 1
        RESPONSE.redirect('manage_editFolderForm')
    
         
    def all_meta_types(self):
        """
        Restricts object types that can be added to a BloggemFolder. 
        Only BloggemItem subclasses can be added to a BloggemFolder.      
        """
        allowedMetaTypes = []
        for bloggemType in bloggemItemTypes:
            if bloggemType[1] <> '':
                allowedMetaTypes.append(bloggemType[1])
        allowedMetaTypes.append('Bloggem Category')
        result = []
        import Products
        for metaType in Products.meta_types:
            if metaType['name'] in allowedMetaTypes:
                    result.append(metaType)
        return result 
    

    #
    # BloggemFolder DTML pages
    #
    index_html = DTMLFile('www/index_html', globals())
    manage_editFolderForm = DTMLFile('www/editFolderForm', globals())

    
    

def addFolderAction(self, title, description=None, meta_title=None, meta_description=None, meta_keywords=None ,id = None, REQUEST=None):
    """Add a BloggemFolder."""
    
    # Check for reserved folder name. 
    checkReservedName(self, id)
    
    self._setObject(id, BloggemFolder(id, title, description, meta_title, meta_description, meta_keywords))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)
    
    
   
#    
# BloggemFolder DTML pages
#

addFolderForm = DTMLFile('www/addFolderForm', globals())     
nameFmtError = DTMLFile('www/nameFmtError', globals()) 

