from Acquisition import Implicit
from Globals import Persistent
from Globals import InitializeClass
from Products.ZCatalog.CatalogAwareness import CatalogAware
from OFS.SimpleItem import Item
from Globals import DTMLFile
from DateTime import DateTime
from OFS.PropertyManager import PropertyManager
import whrandom
import string
from Bloggem import Bloggem
import StructuredText


# Instructions for creating new item types:
#
# BloggemArticle, etc. are members of the BloggemItem class.  These subclasses comprise
# the entries that are displayed in a bloggem.  You can create a new type of entry for 
# a Bloggem by creating a new BloggemItem subclass.  To do so, follow these steps:
#
# 1) Add the "friendly name" and meta_type name to BloggemItemTypes in Bloggem.py
#    The "friendly name" will be used in the url string for filters, and it will be
#    used for the "icon" name in the icon key.
#
# 2) Add an icon to Bloggem/www/gif.  The icon file name should have the format:
#    "icon_[friendly name].gif"
#
#    There is a very good collection of icons available from SourceForge:
#    http://sourceforge.net/projects/icon-collection/
#
# 3) Add an init function to __init__.py.  Use the format of classes above.
#
# 4) Add the class to this file.  (See the bottom of this file.)  The meta_type for your
#    new class should match the meta_type in the BloggemItemTypes list in step 1.
#
# 5) Add an "add form" dtml method to Bloggem/www.  Use "AddArticleForm.dtml" as a guide.  
#    The name of the new form should match the add form referenced in __init__.py.
#    In the new add form, you'll only need to change the heading and the itemType in 
#    the let statement.
#
# 6) Add the name of the add form as specified in __init__ as a global variable in this
#    file.  See the "Factory Forms" section near the bottom of this file.
#
# 7) Add 2 lines in addItemAction() below to handle the creation of the new subclass.
#
# 8) Edit add_edit_main.dtml so that the fields you want are displayed on add/edit.
#    add_edit_main handles all of the add/edit fields for all BloggemItem classes.  If
#    you need to add a new field, you'll need to add the field to this file in 
#    setInstanceVars().
#
# 9) Edit disp_entry_long.dtml so that the proper fields are displayed when displaying
#    the new item type.


class BloggemItem(Implicit, Persistent, PropertyManager, CatalogAware, Item, Bloggem):

    """
    BloggemItem is a Bloggem product. It is the master class for items such as,
    BloggemArticle, BloggemLink and BloggemBook.  BloggemItems are 
    contained by BloggemFolders, and BloggemItems can contain 
    BloggemDiscussions.

    """

    meta_type = 'Bloggem Item'
    
    manage_options = (
        {'label': 'Properties', 'action': 'manage_editItemForm',},
        {'label': 'Preview', 'action': 'disp_entry_long',},
    )
    
    #
    # Init Class Instance
    # 
    def __init__(self, id, itemAttribs, REQUEST):
        """Inits the product with values"""
        
        # Set id, user and create_date
        self.id = id
        self.author = REQUEST.AUTHENTICATED_USER.getUserName()
        self.create_date = DateTime()
        
        setInstanceVars(self, itemAttribs)
        
#        Why was this line here???         
#        self.propertysheets.manage_addProperty('create_date', self.create_date, 'date')


    #
    # Edit Instance
    #
    def manage_editAction(self, REQUEST):
        """Changes the product values"""
        
        # Get form values from REQUEST
        frmAttribs = {}
        for attribPair in REQUEST.form.items():
            frmAttribs[attribPair[0]] = attribPair[1]
        
        # Set instance variables that are provided by edit form
        if frmAttribs.has_key('create_date'):
            self.create_date = DateTime(frmAttribs['create_date'])        
        setInstanceVars(self, frmAttribs)
          
        self._p_changed = 1
        REQUEST.set('change_successful', 'true')
        self.reindex_object()
        return self.manage_editItemForm(self, REQUEST)  
 
    
    # BloggemItem DTML pages 
    index_html = DTMLFile('www/disp_single',  globals()) 
    manage_editItemForm = DTMLFile('www/editItemForm',  globals())

    
def setInstanceVars(self, itemAttribs):
    """
    setInstanceVars applies instance variables for init and manage_edit action.
    The instance variables below comprise all the available attributes for 
    BloggemItems.
    """

    if itemAttribs.has_key('title'):
        self.title = itemAttribs['title']
    if itemAttribs.has_key('content'):
        self.content = itemAttribs['content']
    if itemAttribs.has_key('author'):
        self.author = itemAttribs['author']
    if itemAttribs.has_key('link'):
        self.link = itemAttribs['link']
    if itemAttribs.has_key('bk_author'):
        self.bk_author = itemAttribs['bk_author']
    if itemAttribs.has_key('sticky'):
        self.sticky = itemAttribs['sticky']
#    if itemAttribs.has_key(''):
#        self. = itemAttribs['']
    if itemAttribs.has_key('email_address'):
        self.email_address = itemAttribs['email_address'] 
    if itemAttribs.has_key('email_subject'):
        self.email_subject = itemAttribs['email_subject']
    if itemAttribs.has_key('telephone'):
        self.telephone = itemAttribs['telephone']     
    if itemAttribs.has_key('contact_person'):
        self.contact_person = itemAttribs['contact_person']    
    if itemAttribs.has_key('event_organization'):
        self.event_organization = itemAttribs['event_organization']
    # Address elements
    if itemAttribs.has_key('add_street'):
        self.add_street = itemAttribs['add_street']
    if itemAttribs.has_key('add_street2'):
        self.add_street2 = itemAttribs['add_street2']        
    if itemAttribs.has_key('add_city'):
        self.add_city = itemAttribs['add_city']
    if itemAttribs.has_key('add_state') and itemAttribs['add_state'] <> '--':
        self.add_state = itemAttribs['add_state']
    if itemAttribs.has_key('add_zip'):
        self.add_zip = itemAttribs['add_zip'] 
    # Date Time elements
    if itemAttribs.has_key('date_day') and itemAttribs['date_day'] <> '--' and itemAttribs['date_month'] <> '---':
        # Build date string.
        item_date = itemAttribs['date_year'] + '/' + itemAttribs['date_month'] + '/' + itemAttribs['date_day']
        if itemAttribs.has_key('date_hour') and itemAttribs['date_hour'] <> '--' and itemAttribs['date_minute'] <> '--':
            # Build time string
            item_time = itemAttribs['date_hour'] + ':' + itemAttribs['date_minute'] + ' ' + itemAttribs['date_meridian'] + ' US/' + itemAttribs['date_timezone']
            date_time_str = item_date + ' ' + item_time
            self.item_date_time = DateTime(date_time_str)
            spam = self.item_date_time
    return self

    
#
# Factory Method
#
def addItemAction(self, REQUEST=None, *attribs):
    """Add a BloggemItem to a folder."""
    
    # Create id
    id_integer = `set_id(self)`
    rndm = whrandom.random()
    rndm_string = str(rndm)[2:4]
    id = id_integer + rndm_string
    
    # Create dictionary for form data 
    newAttribs = {}
  
    # Add form items
    for attribPair in REQUEST.form.items():
        newAttribs[attribPair[0]] = attribPair[1]   
        
    # Get type for new item
    item_type = newAttribs['itemType']
        
    # Create object, using itemType to set the class.
    if item_type == 'Bloggem Article':
        self._setObject(id, BloggemArticle(id, newAttribs, REQUEST))
    elif item_type == 'Bloggem Book':
        self._setObject(id, BloggemBook(id, newAttribs, REQUEST))
    elif item_type == 'Bloggem Link':
        self._setObject(id, BloggemLink(id, newAttribs, REQUEST))
    elif item_type == 'Bloggem Organization':
        self._setObject(id, BloggemOrganization(id, newAttribs, REQUEST))
    elif item_type == 'Bloggem Event':
        self._setObject(id, BloggemEvent(id, newAttribs, REQUEST))
    elif item_type == 'Bloggem Email':
        self._setObject(id, BloggemEmail(id, newAttribs, REQUEST))
    elif item_type == 'Bloggem Address':
        self._setObject(id, BloggemAddress(id, newAttribs, REQUEST))
        
       
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)
    
    
    def shortEntry(content, entry_length):
        """
        Takes content from a BloggemItem, shortens it, and adds an STX-compatible
        absolute URL.
        """
        short_content = 'spam'
        return content
    
    
class BloggemArticle(BloggemItem):
    """
    BloggemArticle is a BloggemItem.  BloggemArticles contain text
    content and a title.
    """
    meta_type = 'Bloggem Article'

     
    
class BloggemBook(BloggemItem):
    """
    BloggemBook is a BloggemItem.  BloggemBooks contain text
    content, a title and an URL.
    """
    meta_type = 'Bloggem Book'  



class BloggemLink(BloggemItem):
    """
    BloggemLink is a BloggemItem.  BloggemArticles contain text
    content, a title, and an URL.
    """
    meta_type = 'Bloggem Link'  
   
    
class BloggemEmail(BloggemItem):
    """
    BloggemEmail is a BloggemItem.  BloggemEmails contain a title, 
    description, email address and email subject.
    """
    meta_type = 'Bloggem Email'
    
    
class BloggemOrganization(BloggemItem):
    """
    BloggemOrganization is a BloggemItem.  BloggemOrganizations contain
    a title, description, URL, address (street, town, state, ZIP), and contact
    (person, telephone, email)
    """
    meta_type = 'Bloggem Organization'
    
    
class BloggemEvent(BloggemItem):
    """
    BloggemEvent is a BloggemItem.  BloggemEvents contain a title, description
    contact, address, URL and datetime.
    """
    meta_type = 'Bloggem Event'
    
    
class BloggemAddress(BloggemItem):
    """
    BloggemAddress is a BloggemItem.  BloggemAddresses contain a title, description
    contact, and address.
    """
    meta_type = 'Bloggem Address'    
  
    
#
# Factory Forms
#

addItemForm = DTMLFile('www/addItemForm', globals()) 
addArticleForm = DTMLFile('www/addArticleForm', globals())
addBookForm = DTMLFile('www/addBookForm', globals())
addLinkForm = DTMLFile('www/addLinkForm', globals())
addEmailForm = DTMLFile('www/addEmailForm', globals())
addOrganizationForm = DTMLFile('www/addOrganizationForm', globals())
addEventForm = DTMLFile('www/addEventForm', globals())
addAddressForm = DTMLFile('www/addAddressForm', globals())


# set_id is used to create a unique id for the item.
# TODO: Make this function single threaded.
def set_id(self, new_id=None):
    """
    set_id gets list of used item id's for this Bloggem.  It returns the next
    integer for the item id.  It also adds the used 
    integer to the item_ids list for this Bloggem.
    """

    parent = self.aq_parent
    if new_id is not None:
        return new_id
    
    if parent.meta_type == 'Bloggem':
        id_list = parent.item_ids
        last_id = id_list[-1]
        new_id = last_id + 1
        id_list.append(new_id)
        parent.item_ids = id_list
        return new_id
    
    else:
        new_id = set_id(parent, new_id)
        
    return new_id

